From d99b777987ad61ad80f7e49bc6be35f376ee7560 Mon Sep 17 00:00:00 2001
From: Besnard Jean-Baptiste <jbbesnard@actech-innovation.com>
Date: Wed, 19 Aug 2009 12:09:54 +0200
Subject: [PATCH] Mozilla-plugin, ActiveX : Solving the playlist problem #2862

---
 projects/activex/plugin.h    |   44 ++++++++++++++++++++++++++++++++++++++++++
 projects/mozilla/vlcplugin.h |   44 ++++++++++++++++++++++++++++++++++++++++++
 2 files changed, 88 insertions(+), 0 deletions(-)

diff --git a/projects/activex/plugin.h b/projects/activex/plugin.h
index 867f2d1..333f032 100644
--- a/projects/activex/plugin.h
+++ b/projects/activex/plugin.h
@@ -277,12 +277,55 @@ public:
         if( isPlaying(ex) )
             libvlc_media_player_pause(_p_mplayer,ex);
     }
+    void subitems_sync(libvlc_exception_t *ex)
+    {
+    libvlc_state_t state;
+    int i= 0;
+    libvlc_media_t *media = NULL;
+    libvlc_media_list_t *p_sub = NULL;
+    int sub_item_count = 0;
+    int current_index  = 0;
+    //Waiting for the right state
+    do {
+        i++;
+        state = libvlc_media_player_get_state (_p_mplayer, ex);
+        #if defined (WIN32)
+            Sleep(1);
+        #else
+            usleep(1000);
+        #endif
+    } while( state != libvlc_Error && state != libvlc_Ended && state != libvlc_Playing && i < 500);
+
+    //retrieving the subitems
+    media = libvlc_media_player_get_media(_p_mplayer, ex);
+    p_sub = libvlc_media_subitems(media, ex);
+    //We don't have any subitem
+    if(!p_sub)
+        return;
+    //counting the subitems
+    libvlc_media_list_lock(p_sub);
+    sub_item_count = libvlc_media_list_count(p_sub, ex);
+    //adding subitems to the plugin medialist
+    for(i = 0; i < sub_item_count ; i++)
+    {
+        media = libvlc_media_list_item_at_index( p_sub,i ,ex);
+        playlist_add_extended_untrusted(libvlc_media_get_mrl( media, ex), 0, NULL, ex);
+    }
+    libvlc_media_list_unlock(p_sub);
+    //Saving the playlist file index in playlist
+    current_index = _i_midx;
+    //Switching to next item
+    playlist_next(ex);
+    //Deleting playlist file entry in playlist
+    playlist_delete_item( current_index, ex );
+    }
     void playlist_play(libvlc_exception_t *ex)
     {
         if( !_p_libvlc )
             initVLC();
         if( _p_mplayer||playlist_select(0,ex) )
             libvlc_media_player_play(_p_mplayer,ex);
+        subitems_sync(ex);
     }
     void playlist_play_item(int idx,libvlc_exception_t *ex)
     {
@@ -290,6 +333,7 @@ public:
             initVLC();
         if( playlist_select(idx,ex) )
             libvlc_media_player_play(_p_mplayer,ex);
+        subitems_sync(ex);
     }
     void playlist_stop(libvlc_exception_t *ex)
     {
diff --git a/projects/mozilla/vlcplugin.h b/projects/mozilla/vlcplugin.h
index d14e1a9..c33b521 100644
--- a/projects/mozilla/vlcplugin.h
+++ b/projects/mozilla/vlcplugin.h
@@ -31,6 +31,7 @@
 
 #include <vlc/vlc.h>
 #include <npapi.h>
+#include <unistd.h>
 #include "control/nporuntime.h"
 
 #if !defined(XP_MACOSX) && !defined(XP_UNIX) && !defined(XP_WIN)
@@ -144,15 +145,58 @@ public:
     char *   psz_text;
     char *   psz_target;
 
+    void subitems_sync(libvlc_exception_t *ex)
+    {
+    libvlc_state_t state;
+    int i= 0;
+    libvlc_media_t *media = NULL;
+    libvlc_media_list_t *p_sub = NULL;
+    int sub_item_count = 0;
+    int current_index  = 0;
+    //Waiting for the right state
+    do {
+        i++;
+        state = libvlc_media_player_get_state (libvlc_media_player, ex);
+        #if defined (WIN32)
+            Sleep(1);
+        #else
+            usleep(1000);
+        #endif
+    } while( state != libvlc_Error && state != libvlc_Ended && state != libvlc_Playing && i < 500);
+    //retrieving the subitems
+    media = libvlc_media_player_get_media(libvlc_media_player, ex);
+    p_sub = libvlc_media_subitems(media, ex);
+    //We don't have any subitem
+    if(!p_sub)
+        return;
+    //counting the subitems
+    libvlc_media_list_lock(p_sub);
+    sub_item_count = libvlc_media_list_count(p_sub, ex);
+    //adding subitems to the plugin medialist
+    for(i = 0; i < sub_item_count ; i++)
+    {
+        media = libvlc_media_list_item_at_index( p_sub,i ,ex);
+        playlist_add( libvlc_media_get_mrl( media, ex) , ex);
+    }
+    libvlc_media_list_unlock(p_sub);
+    //Saving the playlist file index in playlist
+    current_index = playlist_index;
+    //Switching to next item
+    playlist_next(ex);
+    //Deleting playlist file entry in playlist
+    playlist_delete_item( current_index, ex );
+    }
     void playlist_play(libvlc_exception_t *ex)
     {
         if( libvlc_media_player||playlist_select(0,ex) )
             libvlc_media_player_play(libvlc_media_player,ex);
+        subitems_sync(ex);
     }
     void playlist_play_item(int idx,libvlc_exception_t *ex)
     {
         if( playlist_select(idx,ex) )
             libvlc_media_player_play(libvlc_media_player,ex);
+        subitems_sync(ex);
     }
     void playlist_stop(libvlc_exception_t *ex)
     {
-- 
1.6.0.4

